@@ -2,7 +2,9 @@
 // { dg-require-fork "" }
 // { dg-require-mkfifo "" }
 
-// Copyright (C) 2003, 2005 Free Software Foundation, Inc.
+// 2004-01-11  Petur Runolfsson  <peturr02@ru.is>
+
+// Copyright (C) 2004, 2005 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -25,49 +27,53 @@
 #include <fstream>
 #include <locale>
 
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include <testsuite_hooks.h>
 
-// libstdc++/13171
+// libstdc++/13582
 void test01()
 {
   bool test __attribute__((unused)) = true;
-  using namespace std;
+  using namespace std; 
   using namespace __gnu_test;
 
-  locale loc_fr(locale("fr_FR"));
   locale loc_en(locale("en_US"));
+  locale loc_fr(locale("fr_FR"));
 
-  const char* name = "tmp_fifo_13171-2";
+  const char* name = "tmp_fifo_13582-2";
   unlink(name);
   mkfifo(name, S_IRWXU);
-  semaphore s1, s2;
-  
+
   int child = fork();
   if (child == 0)
     {
-      filebuf fb;
-      fb.open(name, ios_base::out);
-      fb.sputc('S');
-      fb.pubsync();
-      s1.signal ();
-      s2.wait ();
-      fb.close();
+      filebuf fbout;
+      fbout.open(name, ios_base::out);
+      fbout.sputn("12345", 5);
+      fbout.pubsync();
+      fbout.close();
       exit(0);
     }
 
-  filebuf fb;
-  fb.pubimbue(loc_fr);
-  fb.open(name, ios_base::in);
-  s1.wait ();
-  VERIFY( fb.is_open() );
-  fb.pubimbue(loc_en);
-  filebuf::int_type c = fb.sgetc();
-  fb.close();
-  VERIFY( c == 'S' );
-  s2.signal ();
+  filebuf fbin;
+  fbin.open(name, ios_base::in);
+  filebuf::int_type n = fbin.sbumpc();
+  VERIFY( n == '1' );
+  fbin.pubimbue(loc_en);
+  n = fbin.sbumpc();
+  VERIFY( n == '2' );
+  fbin.pubimbue(loc_fr);
+  n = fbin.sbumpc();
+  VERIFY( n == '3' );
+  n = fbin.sbumpc();
+  VERIFY( n == '4' );
+  n = fbin.sbumpc();
+  VERIFY( n == '5' );
+  n = fbin.sbumpc();
+  VERIFY( n == filebuf::traits_type::eof() );
 }
 
 int main()
