@@ -15,29 +15,28 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 27.4.3 fpos
 
 #include <iterator>
+#include <limits>
 #include <testsuite_hooks.h>
 
-// libstdc++/14320
-int test01()
+void test01()
 {
   using namespace std;
-  bool test __attribute__((unused)) = true;
+  bool test __attribute__((unused)) = true;  
 
-  typedef std::istreambuf_iterator<char>::difference_type Distance;
-  Distance d = 2;
-  Distance e = 3;
-  d *= e;
-  VERIFY( static_cast<int>(d) == 6 );
+  typedef istreambuf_iterator<char>::difference_type Distance;
+  typedef numeric_limits<Distance> Limits;
+
+  VERIFY( Limits::is_specialized );
+  VERIFY( Limits::is_signed );
 }
 
 int main()
 {
   test01();
-  return 0;
 }
